#!/usr/bin/env python
"""
_AWSS3Impl_

Interface for AWS S3 CLI Stage Out Plugin

"""
from __future__ import print_function
from __future__ import division

import os
import logging
from WMCore.Storage.StageOutImpl import StageOutImpl
from WMCore.Storage.Registry import registerStageOutImpl

class AWSS3Impl(StageOutImpl):
    """
    _AWSS3Impl_

    Define the interface for AWS S3 CLI stage out plugin
    """

    def createSourceName(self, protocol, pfn):
        """
        _createSourceName_

        uses pfn

        """
        return "%s" % pfn

    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
        """
        _createStageOutCommand_
        Build an aws s3 copy command
        """
        logging.warning("Warning! AWSS3Impl does not support authMethod handling")

        result = "#!/bin/sh\n"

        copyCommand = "aws s3 cp"

        if options != None:
            copyCommand += " %s " % options
        copyCommand += " %s " % sourcePFN
        copyCommand += " %s \n" % targetPFN

        result += copyCommand

        result += """
            EXIT_STATUS=$?
            echo "aws s3 cp exit status: $EXIT_STATUS"
            if [[ $EXIT_STATUS != 0 ]]; then
               echo "ERROR: Non-zero aws s3 cp Exit status!!!"
               echo "Cleaning up failed file:"
               %s
            fi
            exit $EXIT_STATUS
            """ % self.createRemoveFileCommand(targetPFN)

        return result
    
    def createDebuggingCommand(self, sourcePFN, targetPFN, options=None, checksums=None, authMethod=None, forceMethod=False):
        """
        Debug a failed aws s3 copy command for stageOut, without re-running it,
        providing information on the environment and the certifications

        :sourcePFN: str, PFN of the source file
        :targetPFN: str, destination PFN
        :options: str, additional options for copy command
        :checksums: dict, collect checksums according to the algorithms saved as keys
        """
        # Build the command for debugging purposes
        copyCommand = "aws s3 cp"
        if options != None:
            copyCommand += " %s " % options
        copyCommand += " %s " % sourcePFN
        copyCommand += " %s \n" % targetPFN

        result = self.debuggingTemplate.format(copy_command=copyCommand, source=sourcePFN, destination=targetPFN)
        return result

    def removeFile(self, pfnToRemove):
        """
        _removeFile_
        CleanUp pfn provided
        """
        command = ""
        if pfnToRemove.startswith("s3://"):
            command = "aws s3 rm %s" % pfnToRemove
        elif pfnToRemove.startswith("/"):
            command = "/bin/rm -f %s" % pfnToRemove
        elif os.path.isfile(pfnToRemove):
            command = "/bin/rm -f %s" % os.path.abspath(pfnToRemove)
        self.executeCommand(command)

    def createRemoveFileCommand(self, pfn):
        """
        return the command to delete a file after a failed copy
        """
        if pfn.startswith("s3://"):
            return "aws s3 rm %s" % pfn
        elif pfn.startswith("/"):
            return "/bin/rm -f %s" % pfn
        elif os.path.isfile(pfn):
            return "/bin/rm -f %s" % os.path.abspath(pfn)
        else:
            return ""

registerStageOutImpl("awss3", AWSS3Impl)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
This script can be used to parse the WMCore campaign documents
and extract the equivalent pileup configuration.

Attributes that are used from the campaign documents are:
 * CampaignName: to be defined under the pileup "campaignName" attr
 * Secondaries: to fetch the pileup dataset name, type and expected location

where the pileup document minimal schema is:
{
    'pileupName': "",
    'pileupType': "",
    'expectedRSEs': [],
    'campaigns': [],
    'active': False
}

Example usage is:
==> Construct pileup documents from the production campaigns
python3 createPileupObjects.py --url=https://cmsweb.cern.ch --fout=alan.json

==> Inject the previous pileup doc dump into your test cluster
python3 createPileupObjects.py --url=https://cmsweb-test9.cern.ch --fin=alan.json --inject
"""
import argparse
import json
import logging
import os
import sys
from pprint import pprint

try:
    from WMCore.Services.pycurl_manager import RequestHandler
except ImportError:
    print("WMCore/RequestHandler environment not set. Faking it")
    RequestHandler = None
try:
    from WMCore.Services.ReqMgrAux.ReqMgrAux import ReqMgrAux
except ImportError:
    print("WMCore/ReqMgrAux environment not found. Faking it")
    ReqMgrAux = None


class OptionParser():
    """Class to parse the command line arguments"""

    def __init__(self):
        "User based option parser"
        self.parser = argparse.ArgumentParser(prog='PROG')
        self.parser.add_argument("--fin", action="store",
                                 dest="fin", default="", help="Input file")
        self.parser.add_argument("--fout", action="store",
                                 dest="fout", default="",
                                 help="Output json file with all the pileup configurations created")
        self.parser.add_argument("--url", action="store",
                                 dest="url", default="https://cmsweb-testbed.cern.ch",
                                 help="Server url to fetch and create pileup configurations")
        self.parser.add_argument("--inject", action="store_true",
                                 dest="inject", default=False,
                                 help="Inject pileup documents for real")


def getPUSchema(pileupName, pileupDocs, logger):
    """
    Get a minimalistic pileup data schema, or return an
    existent pileup object.

    :param pileupName: string with the pileup name
    :param pileupDocs: list of pileup documents
    :return: a dictionary with the pileup configuration
    """
    for idx, doc in enumerate(pileupDocs):
        if pileupName == doc["pileupName"]:
            logger.warning("Reusing pileup configuration for pileup: %s", pileupName)
            return pileupDocs.pop(idx)
    pileupDoc = {
        'pileupName': "",
        'pileupType': "",
        'expectedRSEs': [],
        'campaigns': [],
        'active': False}
    return pileupDoc


def parseCampaigns(campDocs, logger):
    """
    Parse campaigns from WMCore and convert them into
    Pileup documents, where:
     * campaigns: is a union of all campaigns using a given pileup.
     * expectedRSEs: is a union of all RSEs across different campaigns.
    """
    pileupDocs = []
    for camp in campDocs:
        # for each secondary dataset, create one pileup document
        for puName, puRSEs in camp.get("Secondaries", {}).items():
            puDoc = getPUSchema(puName, pileupDocs, logger)
            if puName.startswith("/Neutrino") or puName.split("/")[-1] == "PREMIX":
                puType = "premix"
            else:
                puType = "classic"
            puDoc["pileupName"] = puName
            puDoc["pileupType"] = puType
            puDoc["expectedRSEs"].extend(puRSEs)
            if camp.get("CampaignName", ""):
                puDoc["campaigns"].append(camp["CampaignName"])
            # make some of these values unique
            puDoc["expectedRSEs"] = list(set(puDoc["expectedRSEs"]))
            puDoc["campaigns"] = list(set(puDoc.get("campaigns", [])))
            pileupDocs.append(puDoc)
    logger.info("Created %d pileup docs out of %d campaigns.", len(pileupDocs), len(campDocs))
    return pileupDocs


def writePileupDocs(mspileupUrl, puDocs, certDict, logger):
    """
    Create pileup documents in MSPileup
    :param mspileupUrl: string with the MSPileup url
    :param puDocs: list with pileup documents
    :param certDict: dictionary with cert and key reference
    :param logger: logger object
    :return: None
    """
    url = mspileupUrl + "/ms-pileup/data/pileup"
    mgr = RequestHandler()
    headers = {'Content-Type': 'application/json'}
    for doc in puDocs:
        logger.info("Injecting against %s document: %s", url, doc)
        resp = mgr.getdata(url, doc, headers, verb='POST', encode=True, decode=True,
                           ckey=certDict['key'], cert=certDict['cert'])

        logger.debug("Response: %s", resp)
        if resp and resp.get("result", []):
            msg = f"Failed to inject pileup document for {doc['pileupName']}. Error: {resp}"
            logger.critical(msg)


def main():
    """Executes everything"""
    optmgr = OptionParser()
    opts = optmgr.parser.parse_args()
    logger = logging.getLogger('createPileupObjects')
    logger.setLevel(logging.INFO)
    logging.basicConfig()

    # setup proxy/cert
    cert = os.getenv('X509_USER_CERT', '')
    key = os.getenv('X509_USER_KEY', '')
    proxy = os.getenv('X509_USER_PROXY', '')
    if not cert and not proxy:
        logger.error("You need to define the X509 user cert/key or user proxy variables")
        sys.exit(1)
    if not cert and proxy:
        cert = proxy
        key = proxy
    hdict = {'cert': cert, 'key': key, 'pycurl': True}

    if opts.fin:
        fin = opts.fin
        logger.info(f"Reading pileup configuration from file: {fin}")
        with open(fin, 'r') as istream:
            puDocs = json.load(istream)
            logger.info(f"Found {len(puDocs)} pileup objects in the input file.")
    else:
        logger.info("Parsing/fetching pileup objects from ReqMgrAux")
        reqAux = ReqMgrAux(opts.url + "/reqmgr2", hdict, logger=logger)
        campDocs = reqAux.getCampaignConfig("ALL_DOCS")
        logger.info("Retrieved %d campaigns from ReqMgr.", len(campDocs))
        puDocs = parseCampaigns(campDocs, logger)
    if opts.fout:
        logger.info("Saving all %d pileup documents into file: %s", len(puDocs), opts.fout)
        with open(opts.fout, "w") as jo:
            json.dump(puDocs, jo, indent=2, sort_keys=True)

    if opts.inject:
        logger.info("Going to inject %d pileup documents into: %s", len(puDocs), opts.url)
        writePileupDocs(opts.url, puDocs, hdict, logger)
    else:
        logger.info("DRY-RUN: not injecting any pileup documents in MSPileup.")


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
_UpdateFiles_

MySQL implementation of DBS3Buffer.UpdateFiles
"""

from WMCore.Database.DBFormatter import DBFormatter

class UpdateFiles(DBFormatter):
    sql = """UPDATE dbsbuffer_file SET status = :status
             WHERE block_id = (SELECT id FROM dbsbuffer_block WHERE blockname = :block)"""

    def execute(self, blocks, status, conn = None, transaction = False):
        bindVars = []
        for block in blocks:
            bindVars.append({"block": block.getName(), "status": status})

        self.dbi.processData(self.sql, bindVars, conn = conn,
                             transaction = transaction)
        return



#!/usr/bin/env python
"""
_EventBased_

Event based splitting algorithm that will chop a fileset into
a set of jobs based on event counts
"""

from builtins import str
from builtins import range
import logging
import traceback
from math import ceil

from WMCore.DataStructs.Run import Run
from WMCore.JobSplitting.JobFactory import JobFactory
from WMCore.WMBS.File import File


class EventBased(JobFactory):
    """
    Split jobs by number of events
    """

    def algorithm(self, *args, **kwargs):
        """
        _algorithm_

        An event base splitting algorithm.  All available files are split into a
        set number of events per job.
        """
        eventsPerJob = int(kwargs.get("events_per_job", 100))
        eventsPerLumi = int(kwargs.get("events_per_lumi", eventsPerJob))
        getParents = kwargs.get("include_parents", False)
        lheInput = kwargs.get("lheInputFiles", False)
        collectionName = kwargs.get('collectionName', None)
        timePerEvent, sizePerEvent, memoryRequirement = \
            self.getPerformanceParameters(kwargs.get('performance', {}))
        acdcFileList = []
        deterministicPileup = kwargs.get('deterministicPileup', False)

        if eventsPerJob <= 0 or eventsPerLumi <= 0:
            msg = "events_per_job and events_per_lumi must be positive. Their values are: "
            msg += "events_per_job: %d, events_per_lumi: %d" % (eventsPerJob, eventsPerLumi)
            raise RuntimeError(msg)

        if deterministicPileup and self.package == 'WMCore.WMBS':
            getJobNumber = self.daoFactory(classname="Jobs.GetNumberOfJobsPerWorkflow")
            self.nJobs = getJobNumber.execute(workflow=self.subscription.getWorkflow().id)
            logging.info('Creating jobs in DeterministicPileup mode for %s',
                         self.subscription.workflowName())

        # If we have runLumi info, we need to load it from couch
        if collectionName:
            try:
                from WMCore.ACDC.DataCollectionService import DataCollectionService
                couchURL = kwargs.get('couchURL')
                couchDB = kwargs.get('couchDB')
                filesetName = kwargs.get('filesetName')
                collectionName = kwargs.get('collectionName')
                logging.info('Loading ACDC info for collectionName: %s, with filesetName: %s', collectionName,
                             filesetName)
                dcs = DataCollectionService(couchURL, couchDB)
                acdcFileList = dcs.getProductionACDCInfo(collectionName, filesetName)
            except Exception as ex:
                msg = "Exception while trying to load goodRunList\n"
                msg += "Refusing to create any jobs.\n"
                msg += str(ex)
                msg += str(traceback.format_exc())
                logging.error(msg)
                return

        totalJobs = 0

        locationDict = self.sortByLocation()
        for location in locationDict:
            self.newGroup()
            fileList = locationDict[location]
            getRunLumiInformation = False
            for f in fileList:
                if f['lfn'].startswith("MCFakeFile"):
                    # We have one MCFakeFile, then it needs run information
                    getRunLumiInformation = True
                    break
            if getRunLumiInformation:
                if self.package == 'WMCore.WMBS':
                    loadRunLumi = self.daoFactory(classname="Files.GetBulkRunLumi")
                    fileLumis = loadRunLumi.execute(files=fileList)
                    if not fileLumis:
                        logging.warning("Empty fileLumis dict for workflow %s, subs %s.",
                                        self.subscription.workflowName(), self.subscription['id'])
                    for f in fileList:
                        lumiDict = fileLumis.get(f['id'], {})
                        for run in lumiDict:
                            f.addRun(run=Run(run, *lumiDict[run]))

            for f in fileList:
                currentEvent = f['first_event']
                eventsInFile = f['events']
                runs = list(f['runs'])
                # We got the runs, clean the file.
                f['runs'] = set()

                if getParents:
                    parentLFNs = self.findParent(lfn=f['lfn'])
                    for lfn in parentLFNs:
                        parent = File(lfn=lfn)
                        f['parents'].add(parent)

                if acdcFileList:
                    totalJobs = self.createACDCJobs(f, acdcFileList, timePerEvent,
                                                    sizePerEvent, memoryRequirement,
                                                    lheInput, eventsPerJob, eventsPerLumi,
                                                    deterministicPileup, totalJobs)
                    continue
                if not f['lfn'].startswith("MCFakeFile"):
                    # there might be files with 0 event that still have to be processed
                    if eventsInFile == 0:
                        self.newJob(name=self.getJobName(length=totalJobs))
                        self.currentJob.addFile(f)
                        # Do not set LastEvent
                        self.currentJob["mask"].setMaxAndSkipEvents(None, currentEvent)
                        self.currentJob.addResourceEstimates(jobTime=0,
                                                             memory=memoryRequirement,
                                                             disk=0)
                        if deterministicPileup:
                            self.currentJob.addBaggageParameter("skipPileupEvents", (self.nJobs - 1) * eventsPerJob)
                        totalJobs += 1
                        logging.info("Job created for 0-event input file with %s", self.currentJob)
                    # Very very uncommon in production, but it has real input dataset
                    while eventsInFile:
                        self.newJob(name=self.getJobName(length=totalJobs))
                        self.currentJob.addFile(f)
                        if eventsInFile >= eventsPerJob:
                            jobTime = eventsPerJob * timePerEvent
                            diskRequired = eventsPerJob * sizePerEvent
                            self.currentJob["mask"].setMaxAndSkipEvents(eventsPerJob - 1, currentEvent)
                        else:
                            jobTime = eventsInFile * timePerEvent
                            diskRequired = eventsInFile * sizePerEvent
                            self.currentJob["mask"].setMaxAndSkipEvents(eventsInFile - 1, currentEvent)
                            eventsInFile = eventsPerJob
                        self.currentJob.addResourceEstimates(jobTime=jobTime,
                                                             memory=memoryRequirement,
                                                             disk=diskRequired)
                        if deterministicPileup:
                            self.currentJob.addBaggageParameter("skipPileupEvents", (self.nJobs - 1) * eventsPerJob)

                        eventsInFile -= eventsPerJob
                        currentEvent += eventsPerJob
                        totalJobs += 1
                        logging.debug("Job created for real input with %s", self.currentJob)
                else:
                    # This assumes there's only one run which is the case for MC
                    lumis = runs[0].lumis
                    (firstLumi, lastLumi) = (min(lumis), max(lumis))
                    currentLumi = firstLumi
                    lumisPerJob = int(ceil(float(eventsPerJob) / eventsPerLumi))

                    while eventsInFile:
                        self.newJob(name=self.getJobName(length=totalJobs))
                        self.currentJob.addFile(f)
                        self.currentJob.addBaggageParameter("lheInputFiles", lheInput)

                        # Limit the number of events to a unsigned 32bit int
                        if (currentEvent + eventsPerJob - 1) > (2 ** 32 - 1) and \
                                        (currentEvent + eventsInFile) > (2 ** 32 - 1):
                            currentEvent = 1

                        if eventsInFile >= eventsPerJob:
                            jobTime = eventsPerJob * timePerEvent
                            diskRequired = eventsPerJob * sizePerEvent
                            # Alan on 16/Apr/2019: inclusiveMask must be a real inclusiveMask, thus
                            # FirstEvent/FirstLumi and LastEvent/LastLumi are also processed by the job
                            self.currentJob["mask"].setMaxAndSkipEvents(eventsPerJob - 1, currentEvent)
                            self.currentJob["mask"].setMaxAndSkipLumis(lumisPerJob - 1, currentLumi)
                        else:
                            jobTime = eventsInFile * timePerEvent
                            diskRequired = eventsInFile * sizePerEvent
                            lumisPerJob = int(ceil(float(eventsInFile) / eventsPerLumi))
                            self.currentJob["mask"].setMaxAndSkipEvents(eventsInFile - 1, currentEvent)
                            self.currentJob["mask"].setMaxAndSkipLumis(lumisPerJob - 1, currentLumi)
                            eventsInFile = eventsPerJob

                        self.currentJob.addResourceEstimates(jobTime=jobTime,
                                                             memory=memoryRequirement,
                                                             disk=diskRequired)
                        if deterministicPileup:
                            self.currentJob.addBaggageParameter("skipPileupEvents", (self.nJobs - 1) * eventsPerJob)

                        eventsInFile -= eventsPerJob
                        currentEvent += eventsPerJob
                        currentLumi += lumisPerJob
                        totalJobs += 1
                        logging.info("Job created with mask: %s", self.currentJob['mask'])

        return

    def createACDCJobs(self, fakeFile, acdcFileInfo, timePerEvent, sizePerEvent,
                       memoryRequirement, lheInputOption, eventsPerJob,
                       eventsPerLumi, deterministicPileup, totalJobs=0):
        """
        _createACDCJobs_

        Create ACDC production jobs, these are treated differently
        since it is an exact copy of the failed jobs.
        """
        lumisPerJob = 0  # calculated a bit beyond
        for acdcFile in acdcFileInfo:
            if fakeFile['lfn'] == acdcFile['lfn']:
                eventsToRun = acdcFile["events"]
                currentEvent = acdcFile["first_event"]
                acdcFile["lumis"] = sorted(acdcFile["lumis"])
                while eventsToRun > 0:
                    ### WARNING: it assumes there will NOT be splitting changes between
                    # the original and the ACDC workflow
                    # Lumis to recover are not necessarily sequential
                    acdcFile["lumis"] = acdcFile["lumis"][lumisPerJob:]
                    currentLumi = acdcFile["lumis"][0]
                    self.newJob(name=self.getJobName(length=totalJobs))
                    self.currentJob.addFile(fakeFile)
                    self.currentJob.addBaggageParameter("lheInputFiles", lheInputOption)
                    # Limit the number of events to a unsigned 32bit int
                    if (currentEvent + eventsPerJob) > (2 ** 32 - 1):
                        currentEvent = 1
                    if eventsToRun >= eventsPerJob:
                        self.currentJob["mask"].setMaxAndSkipEvents(eventsPerJob - 1, currentEvent)
                    else:
                        self.currentJob["mask"].setMaxAndSkipEvents(eventsToRun - 1, currentEvent)
                    if fakeFile['lfn'].startswith("MCFakeFile"):
                        # either a shorter last job or a normal sized one
                        lumisPerJob = int(ceil(float(min(eventsToRun, eventsPerJob)) / eventsPerLumi))
                        # I don't like it! but if we want to reduce the memory footprint, we need
                        # to keep merging MCFakeFiles at DataCollection level. Which requires a dirty
                        # lumi check - for sequential lumis - in here
                        while True:
                            setLumis = set(range(currentLumi, currentLumi + lumisPerJob))
                            if setLumis.issubset(set(acdcFile["lumis"])):
                                break
                            else:
                                lumisPerJob -= 1
                        self.currentJob["mask"].setMaxAndSkipLumis(lumisPerJob - 1, currentLumi)
                    jobTime = eventsPerJob * timePerEvent
                    diskRequired = eventsPerJob * sizePerEvent
                    self.currentJob.addResourceEstimates(jobTime=jobTime,
                                                         memory=memoryRequirement,
                                                         disk=diskRequired)
                    if deterministicPileup:
                        self.currentJob.addBaggageParameter("skipPileupEvents", (self.nJobs - 1) * eventsPerJob)
                    logging.info("ACDC job created with mask: %s", self.currentJob['mask'])
                    eventsToRun -= eventsPerJob
                    currentEvent += eventsPerJob
                    currentLumi += lumisPerJob
                    totalJobs += 1
        return totalJobs



#!/usr/bin/env python

"""
_Step.Executor.DeleteFiles_

Implementation of an Executor for a Delete step

"""
from __future__ import print_function

from future.utils import viewitems

import signal

from WMCore.Storage.DeleteMgr import DeleteMgr, DeleteMgrError
from WMCore.Storage.FileManager import DeleteMgr as NewDeleteMgr
from WMCore.WMExceptions import WM_JOB_ERROR_CODES
from WMCore.WMSpec.Steps.Executor import Executor
from WMCore.WMSpec.Steps.Executors.LogArchive import Alarm, alarmHandler


class DeleteFiles(Executor):
    """
    A step run to clean up the unmerged files in a job

    """

    def pre(self, emulator=None):
        """
        _pre_

        Pre execution checks

        """

        # Are we using an emulator?
        if emulator is not None:
            return emulator.emulatePre(self.step)

        self.logger.info("Steps.Executors.%s.pre called", self.__class__.__name__)
        return None

    def execute(self, emulator=None):
        """
        _execute_

        """

        # Are we using emulators again?
        if emulator is not None:
            return emulator.emulate(self.step, self.job)

        self.logger.info("Steps.Executors.%s.execute called", self.__class__.__name__)
        self.logger.info("Step set to numberOfRetries: %s, retryDelay: %s",
                         self.step.retryCount, self.step.retryDelay)

        # Look!  I can steal from StageOut
        # DeleteMgr uses the same manager structure as StageOutMgr

        overrides = {}
        if hasattr(self.step, 'override'):
            overrides = self.step.override.dictionary_()
        # Wait up to 5min for a single file deletion
        overrides.setdefault('waitTime', 300)

        self.logger.info("Step with the following overrides:")
        for keyName, value in viewitems(overrides):
            self.logger.info("    %s : %s", keyName, value)

        # Pull out StageOutMgr Overrides
        # switch between old stageOut behavior and new, fancy stage out behavior
        useNewStageOutCode = False
        if 'newStageOut' in overrides and overrides.get('newStageOut'):
            useNewStageOutCode = True

        stageOutCall = {}
        stageOutCall['logger'] = self.logger
        if "command" in overrides and "option" in overrides \
                and "phedex-node" in overrides \
                and "lfn-prefix" in overrides:
            stageOutCall['command'] = overrides.get('command')
            stageOutCall['option'] = overrides.get('option')
            stageOutCall['phedex-node'] = overrides.get('phedex-node')
            stageOutCall['lfn-prefix'] = overrides.get('lfn-prefix')

        # naw man, this is real
        # iterate over all the incoming files
        if not useNewStageOutCode:
            # old style
            manager = DeleteMgr(**stageOutCall)
            manager.numberOfRetries = self.step.retryCount
            manager.retryPauseTime = self.step.retryDelay
        else:
            # new style
            self.logger.critical("DeleteFiles IS USING NEW STAGEOUT CODE")
            manager = NewDeleteMgr(retryPauseTime=self.step.retryDelay,
                                   numberOfRetries=self.step.retryCount,
                                   **stageOutCall)

        # This is where the deleted files go
        filesDeleted = []

        for fileDict in self.job['input_files']:
            self.logger.debug("Deleting LFN: %s", fileDict.get('lfn'))
            fileForTransfer = {'LFN': fileDict.get('lfn'),
                               'PFN': None,  # PFNs are assigned in the Delete Manager
                               'PNN': None,  # PNN is assigned in the delete manager
                               'StageOutCommand': None}
            if self.deleteOneFile(fileForTransfer, manager, overrides['waitTime']):
                filesDeleted.append(fileForTransfer)

        # Alan: I do not get why we would have two sets of files to be deleted!
        if hasattr(self.step, 'filesToDelete'):
            # files from the configTree to be deleted
            for k, v in viewitems(self.step.filesToDelete.dictionary_()):
                if k.startswith('file'):
                    self.logger.info("Deleting LFN: %s", v)
                    fileForTransfer = {'LFN': v,
                                       'PFN': None,
                                       'PNN': None,
                                       'StageOutCommand': None}
                    if self.deleteOneFile(fileForTransfer, manager, overrides['waitTime']):
                        filesDeleted.append(fileForTransfer)

        if not filesDeleted:
            raise DeleteMgrError(WM_JOB_ERROR_CODES[60313])

        # Now we've got to put things in the report
        for fileDict in filesDeleted:
            self.report.addRemovedCleanupFile(**fileDict)

    def deleteOneFile(self, fileForTransfer, manager, waitTime):
        signal.signal(signal.SIGALRM, alarmHandler)
        signal.alarm(waitTime)

        try:
            manager(fileToDelete=fileForTransfer)
            # Afterwards, the file should have updated info.
            return fileForTransfer

        except Alarm:
            msg = "Indefinite hang during stageOut of logArchive"
            self.logger.error(msg)
        except Exception as ex:
            self.logger.error("General failure in StageOut for DeleteFiles. Error: %s", str(ex))

        signal.alarm(0)

    def post(self, emulator=None):
        """
        _post_

        Post execution checkpointing

        """
        # Another emulator check
        if emulator is not None:
            return emulator.emulatePost(self.step)

        self.logger.info("Steps.Executors.%s.post called", self.__class__.__name__)
        return None



from __future__ import print_function, division
#filtered list which only send list information for DAS query
DAS_RESULT_FILTER = {"filter_list": ["RequestName", "PrepID", "CMSSWVersion", "GlobalTag"]}
